@@ -2,7 +2,10 @@
 # See LICENSE.txt for terms of redistribution and use.
 
 import copy
-import urllib
+try:
+    from urllib.parse import quote, unquote
+except ImportError:
+    from urllib import quote, unquote  # NOQA
 from datetime import datetime, timedelta
 from calendar import timegm
 from email.utils import formatdate
@@ -14,13 +17,16 @@ class Cookie(object):
 
     See rfc2109, HTTP State Management Mechanism
 
-    >>> c = Cookie('session_id', 'abc123')
-    >>> c.path = '/cgi-bin'
-    >>> c.domain = '.example.org'
-    >>> c.path
-    '/cgi-bin'
-    >>> print str(c)
-    session_id=abc123;Domain=.example.org;Path=/cgi-bin;Version=1
+    Example::
+
+        >>> from fresco.cookie import Cookie
+        >>> c = Cookie('session_id', 'abc123')
+        >>> c.path = '/cgi-bin'
+        >>> c.domain = '.example.org'
+        >>> c.path
+        '/cgi-bin'
+        >>> str(c)
+        'session_id=abc123;Domain=.example.org;Path=/cgi-bin;Version=1'
     """
     attributes = [
         ("Comment", "comment"),
@@ -57,7 +63,7 @@ class Cookie(object):
         Returns a string representation of the cookie in the format, eg
         ``session_id=abc123;Path=/cgi-bin;Domain=.example.com;Version=1``
         """
-        cookie = ['%s=%s' % (self.name, urllib.quote(str(self.value)))]
+        cookie = ['%s=%s' % (self.name, quote(str(self.value)))]
         for cookie_name, att_name in self.attributes:
             value = getattr(self, att_name, None)
             if value is not None:
@@ -98,12 +104,12 @@ def expire_cookie(cookie_or_name, *args, **kwargs):
     Synopsis::
 
         >>> from fresco import Response
-        >>> def view(request):
+        >>> def view():
         ...     return Response(set_cookie=expire_cookie('X', path='/'))
         ...
         >>> from fresco import FrescoApp
         >>> with FrescoApp().requestcontext() as c:
-        ...     print view(c.request).get_header('Set-Cookie')
+        ...     print view().get_header('Set-Cookie')
         ...
         X=;Expires=Tue, 01 Jan 1980 00:00:00 -0000;Max-Age=0;Path=/;Version=1
     """
@@ -121,7 +127,7 @@ def expire_cookie(cookie_or_name, *args, **kwargs):
     )
 
 
-def parse_cookie_header(cookie_string, unquote=urllib.unquote):
+def parse_cookie_header(cookie_string, unquote=unquote):
     """
     Return a list of Cookie objects read from the request headers.
 
