@@ -21,9 +21,6 @@ from fresco.options import Options
 
 __all__ = ('FrescoApp', 'urlfor')
 
-#: Class to use to instantiate request objects
-request_class = Request
-
 
 class FrescoApp(RouteCollection):
     """\
@@ -36,6 +33,9 @@ class FrescoApp(RouteCollection):
     #: A stdlib logger object, or None
     logger = None
 
+    #: Class to use to instantiate request objects
+    request_class = Request
+
     def __init__(self, views=None, path=None):
 
         super(FrescoApp, self).__init__()
@@ -87,7 +87,7 @@ class FrescoApp(RouteCollection):
         for route in self.__routes__:
             s += '    ' + str(route)
         s += '>'
-        return  s
+        return s
 
     def view(self, routecollection=None, request=None, path=None, method=None):
         routecollection = routecollection or self
@@ -118,28 +118,26 @@ class FrescoApp(RouteCollection):
                     traversal.kwargs[k] = v
 
                 environ['wsgiorg.routing_args'] = (traversal.args,
-                                                traversal.kwargs)
-                context.view_self = getattr(route.original_view, '__self__',
-                                            None)
+                                                   traversal.kwargs)
+                view = route.getview()
+                context.view_self = getattr(view, '__self__', None)
 
                 if self.logger:
                     self.logger.info("matched route: %s %r => %r",
-                                     request.method,
-                                     path, route.original_view)
+                                     request.method, path, view)
 
-                context.route_collections_traversed = \
-                            traversal.collections_traversed
-                self.route_matched.send(self, route=route, view=route.view,
+                context.route_traversal = traversal
+                self.route_matched.send(self, route=route, view=view,
                                         request=request)
 
-                response = route.view(*traversal.args, **traversal.kwargs)
+                response = route.getdecoratedview()(*traversal.args,
+                                                    **traversal.kwargs)
             except ResponseException as e:
                 if e.is_final:
                     return e.response
                 error_response = error_response or e.response
             else:
-                self.view_finished.send(self, view=route.view,
-                                        response=response)
+                self.view_finished.send(self, view=view, response=response)
                 return response
 
         # A route was matched, but an error was returned
@@ -184,7 +182,8 @@ class FrescoApp(RouteCollection):
         self._wsgi_app = self._make_wsgi_app()
 
     def _raw_wsgi_app(self, environ, start_response):
-        return self.view(self, request_class(environ))(environ, start_response)
+        return self.view(self, self.request_class(environ))(environ,
+                                                            start_response)
 
     def _make_wsgi_app(self):
         app = self._raw_wsgi_app
@@ -218,12 +217,18 @@ class FrescoApp(RouteCollection):
         fragment = kwargs.pop('_fragment', None)
 
         request = context.request
-        traversal = context.get('route_collections_traversed') or [(self, '')]
+        traversal = context.get('route_traversal')
+        if traversal and traversal.collections_traversed:
+            collections_traversed = traversal.collections_traversed
+        else:
+            collections_traversed = [(self, '')]
+
         exc = None
-        for collection, base_path in traversal[::-1]:
+        for collection, base_path in collections_traversed[::-1]:
             try:
                 path = base_path + \
-                        collection.pathfor(viewspec, *args, **kwargs)
+                        collection.pathfor(viewspec, request=request,
+                                           *args, **kwargs)
             except RouteNotFound as e:
                 exc = e
                 continue
@@ -307,7 +312,7 @@ class FrescoApp(RouteCollection):
             environ['QUERY_STRING'] = url.query
 
         environ.update(env_overrides)
-        context.push(request=request_class(environ), app=self)
+        context.push(request=self.request_class(environ), app=self)
         yield context
         context.pop()
 
