@@ -48,27 +48,27 @@ class MultiDict(MutableMapping):
             1. From a sequence of ``(key, value)`` pairs::
 
                 >>> from fresco.multidict import MultiDict
-                >>> MultiDict([('a', 1), ('a', 2)])
-                MultiDict([('a', 1), ('a', 2)])
+                >>> MultiDict([('a', 1), ('a', 2)])  # doctest: +ELLIPSIS
+                MultiDict(...)
 
             2. Initialized from another MultiDict::
 
                 >>> from fresco.multidict import MultiDict
                 >>> d = MultiDict([('a', 1), ('a', 2)])
-                >>> MultiDict(d)
-                MultiDict([('a', 1), ('a', 2)])
+                >>> MultiDict(d)  # doctest: +ELLIPSIS
+                MultiDict(...)
 
             3. Initialized from a regular dict::
 
                 >>> from fresco.multidict import MultiDict
-                >>> MultiDict({'a': 1})
-                MultiDict([('a', 1)])
+                >>> MultiDict({'a': 1})  # doctest: +ELLIPSIS
+                MultiDict(...)
 
             4. From keyword arguments::
 
                 >>> from fresco.multidict import MultiDict
-                >>> MultiDict(a=1)
-                MultiDict([('a', 1)])
+                >>> MultiDict(a=1)  # doctest: +ELLIPSIS
+                MultiDict(...)
 
         """
         if len(args) > 1:
@@ -137,7 +137,7 @@ class MultiDict(MutableMapping):
         return self._dict.get(key, [])
 
     def copy(self):
-        """
+        """\
         Return a shallow copy of the dictionary::
 
             >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
@@ -150,7 +150,7 @@ class MultiDict(MutableMapping):
         return self.__class__(self)
 
     def fromkeys(cls, seq, value=None):
-        """
+        """\
         Create a new MultiDict with keys from seq and values set to value.
 
         Example::
@@ -171,12 +171,12 @@ class MultiDict(MutableMapping):
     fromkeys = classmethod(fromkeys)
 
     def items(self):
-        """
+        """\
         Return a list of ``(key, value)`` tuples. Only the first ``(key,
         value)`` is returned where keys have multiple values::
 
             >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
-            >>> d.items()
+            >>> list(d.items())
             [('a', 1), ('b', 3)]
         """
         seen = set()
@@ -187,30 +187,30 @@ class MultiDict(MutableMapping):
             seen.add(k)
 
     def listitems(self):
-        """
+        """\
         Like ``items``, but returns lists of values::
 
             >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
-            >>> d.listitems()
+            >>> list(d.listitems())
             [('a', [1, 2]), ('b', [3])]
         """
         for k in self.keys():
             yield k, self._dict[k]
 
     def allitems(self):
-        """
-        Return a list of ``(key, value)`` pairs for each item in the MultiDict.
+        """\
+        Return ``(key, value)`` pairs for each item in the MultiDict.
         Items with multiple keys will have multiple key-value pairs returned::
 
             >>> from fresco.multidict import MultiDict
             >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
-            >>> d.allitems()
+            >>> list(d.allitems())
             [('a', 1), ('a', 2), ('b', 3)]
         """
         return iter(self._order)
 
     def keys(self):
-        """
+        """\
         Return dictionary keys. Each key is returned only once, even if
         multiple values are present.
 
@@ -222,27 +222,27 @@ class MultiDict(MutableMapping):
         return (k for k, v in self.items())
 
     def values(self):
-        """
+        """\
         Return values from the dictionary. Where keys have multiple values,
         only the first is returned::
 
             >>> from fresco.multidict import MultiDict
             >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
-            >>> d.values()
+            >>> list(d.values())
             [1, 3]
         """
         return (v for k, v in self.items())
 
     def listvalues(self):
-        """
-        Return a list of values. Each item returned is a list of values
-        associated with a single key.
+        """\
+        Return an iterator over lists of values. Each item will be the list of
+        values associated with a single key.
 
         Example usage::
 
             >>> from fresco.multidict import MultiDict
             >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
-            >>> d.listvalues()
+            >>> list(d.listvalues())
             [[1, 2], [3]]
         """
         return (self._dict[k] for k in self.keys())
@@ -401,7 +401,7 @@ class MultiDict(MutableMapping):
         """
         ``__repr__`` representation of object
         """
-        return '%s(%r)' % (self.__class__.__name__, self.allitems())
+        return '%s(%r)' % (self.__class__.__name__, list(self.allitems()))
 
     def __str__(self):
         """
