@@ -101,12 +101,9 @@ class Request(object):
                     )
                 )
             else:
-                self._form = MultiDict(
-                    parse_querystring(
-                        self.environ.get('QUERY_STRING', ''),
-                        self.charset
-                    )
-                )
+                query = environ_to_unicode(self.environ.get('QUERY_STRING',
+                                                            ''))
+                self._form = MultiDict(parse_querystring(query, self.charset))
         return self._form
 
     @property
@@ -132,7 +129,7 @@ class Request(object):
             >>> from fresco import FrescoApp
             >>> with FrescoApp().requestcontext('/?animal=moose') as c:
             ...     c.request.query['animal']
-            'moose'
+            u'moose'
 
         This property always returns querystring data, regardless of the
         request method used.
@@ -225,7 +222,7 @@ class Request(object):
 
     @property
     def path(self):
-        """
+        """\
         Return the path component of the requested URL
         """
         return environ_to_unicode(self.environ.get('SCRIPT_NAME', '') +
@@ -233,14 +230,14 @@ class Request(object):
 
     @property
     def url(self):
-        """
+        """\
         Return the full URL, including query parameters.
         """
         return urlunparse(self.parsed_url)
 
     @property
     def application_url(self):
-        """
+        """\
         Return the base URL of the WSGI application (up to SCRIPT_NAME but not
         including PATH_INFO or query information).
 
@@ -251,7 +248,7 @@ class Request(object):
             ...                                 SCRIPT_NAME='/animals',
             ...                                 PATH_INFO='/lion.html') as c:
             ...     c.request.application_url
-            'http://example.com/animals'
+            u'http://example.com/animals'
         """
         scheme, netloc, path, params, query, frag = self.parsed_url
 
@@ -273,8 +270,8 @@ class Request(object):
             >>> app = FrescoApp()
             >>> with app.requestcontext(
             ...          'https://example.com/animals/view?name=lion') as c:
-            ...     c.request.parsed_url
-            ('https', 'example.com', '/animals/view', '', 'name=lion', '')
+            ...     c.request.parsed_url  # doctest: +ELLIPSIS
+            ParseResult(scheme=u'https', netloc=u'example.com', ...)
 
         Components are returned as unicode strings
         """
@@ -406,6 +403,7 @@ class Request(object):
                             ``path``
         :param kwargs:      Any remaining keyword arguments are appended to the
                             querystring
+        :rtype: str
 
         All arguments (other than ``query``, see examples below) may be
         byte strings or unicode strings (native strings in python 3). Where
@@ -423,56 +421,55 @@ class Request(object):
             ...     request = c.request
 
             >>> request.make_url(query=request.query)
-            'http://example.com/fruitsalad/banana?cream=n'
+            u'http://example.com/fruitsalad/banana?cream=n'
 
         Replace the URL scheme::
 
             >>> request.make_url(scheme='ftp')
-            'ftp://example.com/fruitsalad/banana'
+            u'ftp://example.com/fruitsalad/banana'
 
         Replace the entire path::
 
             >>> request.make_url(path='/sausages')
-            'http://example.com/sausages'
+            u'http://example.com/sausages'
 
         Replace just the path_info portion::
 
             >>> request.make_url(PATH_INFO='/sausages')
-            'http://example.com/fruitsalad/sausages'
+            u'http://example.com/fruitsalad/sausages'
 
         Add a query string::
 
             request.make_url(query={'portions': '2'}, cherries='n')
-            'http://example.com/fruitsalad/banana?portions=2;cherries='n'
+            u'http://example.com/fruitsalad/banana?portions=2;cherries='n'
 
         The ``query`` argument can take a dictionary, a list of ``(name,
         value)`` pairs or any other type convertable to a MultiDict::
 
             >>> request.make_url(query='a=tokyo&b=milan')
-            'http://example.com/fruitsalad/banana?a=tokyo&b=milan'
+            u'http://example.com/fruitsalad/banana?a=tokyo&b=milan'
 
             >>> request.make_url(query={'a': 'tokyo', 'b': 'milan'})
-            'http://example.com/fruitsalad/banana?a=tokyo;b=milan'
+            u'http://example.com/fruitsalad/banana?a=tokyo;b=milan'
 
             >>> request.make_url(query=[('a', 'tokyo'),
             ...                         ('b', 'milan'),
             ...                         ('b', 'paris')])
-            'http://example.com/fruitsalad/banana?a=tokyo;b=milan;b=paris'
+            u'http://example.com/fruitsalad/banana?a=tokyo;b=milan;b=paris'
 
         If a relative path is passed, the returned URL is joined to the old in
         the same way as a web browser would interpret a relative HREF in a
         document at the current location::
 
             >>> request.make_url(path='kiwi')
-            'http://example.com/fruitsalad/kiwi'
+            u'http://example.com/fruitsalad/kiwi'
 
             >>> request.make_url(path='../strawberry')
-            'http://example.com/strawberry'
+            u'http://example.com/strawberry'
 
             >>> request.make_url(path='../../../plum')
-            'http://example.com/plum'
+            u'http://example.com/plum'
 
-        :rtype: str
         """
         url = []
         parsed_url = self.parsed_url
