@@ -11,15 +11,38 @@ from fresco.response import Response
 from fresco.requestcontext import context
 from fresco.util.urls import join_path
 
-HTTP_METHODS = GET, HEAD, POST, PUT, DELETE = \
-               'GET', 'HEAD', 'POST', 'PUT', 'DELETE'
+RFC2518_METHODS = \
+    PROPFIND, PROPPATCH, MKCOL, COPY, MOVE, LOCK, UNLOCK = \
+    'PROPFIND', 'PROPPATCH', 'MKCOL', 'COPY', 'MOVE', 'LOCK', 'UNLOCK'
 
-ALL_METHODS = HTTP_METHODS
+RFC2616_METHODS = \
+    GET, HEAD, POST, PUT, DELETE, OPTIONS, TRACE, CONNECT = \
+    'GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'OPTIONS', 'TRACE', 'CONNECT'
 
+RFC3253_METHODS = \
+    VERSION_CONTROL, REPORT, CHECKOUT, CHECKIN, UNCHECKOUT, MKWORKSPACE,\
+    UPDATE, LABEL, MERGE, BASELINE_CONTROL, MKACTIVITY = \
+    'VERSION-CONTROL', 'REPORT', 'CHECKOUT', 'CHECKIN', 'UNCHECKOUT', \
+    'MKWORKSPACE', 'UPDATE', 'LABEL', 'MERGE', 'BASELINE-CONTROL', \
+    'MKACTIVITY',
 
-__all__ = HTTP_METHODS + ('ALL_METHODS', 'Pattern', 'Route', 'DelegateRoute',
-                          'RouteCollection', 'FormArg', 'QueryArg',
-                          'CookieArg', 'routefor')
+RFC3648_METHODS = ORDERPATCH, = ('ORDERPATCH',)
+
+RFC3744_METHODS = ACL, = ('ACL',)
+
+RFC5323_METHODS = SEARCH, = ('SEARCH',)
+
+RFC5789_METHODS = PATCH, = ('PATCH',)
+
+ALL_METHODS = HTTP_METHODS = (RFC2518_METHODS + RFC2616_METHODS +
+                              RFC3253_METHODS + RFC3648_METHODS +
+                              RFC5323_METHODS + RFC5789_METHODS)
+
+__all__ = ('ALL_METHODS', 'Pattern', 'Route', 'DelegateRoute',
+           'RouteCollection', 'FormArg', 'QueryArg',
+           'CookieArg', 'routefor')
+
+__all__ += tuple(method.replace('-', '_') for method in ALL_METHODS)
 
 
 #: Encapsulate a pattern match on a path
@@ -32,6 +55,9 @@ RouteTraversal = namedtuple('RouteTraversal',
                              'collections_traversed'])
 
 
+_marker = object()
+
+
 class URLGenerationError(Exception):
     """\
     Was not possible to generate the requested URL.
@@ -74,6 +100,12 @@ class Pattern(object):
         """
         raise NotImplementedError()
 
+    def path_argument_info(self):
+        """
+        Return information about the arguments required for ``pathfor``
+        """
+        raise NotImplementedError()
+
     def add_prefix(self, prefix):
         """
         Return a copy of the pattern with the given string prepended
@@ -111,7 +143,7 @@ class Converter(object):
 
     def from_string(self, s):
         """
-        Convert string argument ``a`` to the target object representation,
+        Convert string argument ``s`` to the target object representation,
         whatever that may be.
         """
         raise NotImplementedError
@@ -154,7 +186,9 @@ class StrConverter(Converter):
 
 class AnyConverter(StrConverter):
     """
-    Match any one of the given string options.
+    Match any one of the given string options. Example::
+
+        "/<lang:any('fr', 'en', 'de')>"
     """
 
     pattern = r'[+-]?\d+'
@@ -190,10 +224,13 @@ class MatchAllURLsPattern(Pattern):
 
     def pathfor(self, *args, **kwargs):
         assert not args and not kwargs, \
-                "MatchAllURLsPattern does not support URL arguments"
+            "MatchAllURLsPattern does not support URL arguments"
 
         return self.path, (), {}
 
+    def path_argument_info(self):
+        return (), {}
+
     def add_prefix(self, prefix):
         return self.__class__(join_path(prefix, self.path))
 
@@ -209,8 +246,8 @@ class ExtensiblePattern(Pattern):
 
         >>> from pprint import pprint
         >>> p = ExtensiblePattern(r"/<:str>/<year:int>/<title:str>")
-        >>> pprint(p.match('/archive/1999/blah'))
-        (('archive',), {'title': 'blah', 'year': 1999})
+        >>> pprint(p.match('/archive/1999/blah')) # doctest: +ELLIPSIS
+        PathMatch(...)
 
     Patterns are split on slashes into components. A component can either be a
     literal part of the path, or a pattern component in the form::
@@ -236,16 +273,16 @@ class ExtensiblePattern(Pattern):
     Some examples::
 
         >>> p = ExtensiblePattern(r"/images/<:path>")
-        >>> p.match('/images/thumbnails/02.jpg')
-        (('thumbnails/02.jpg',), {})
+        >>> p.match('/images/thumbnails/02.jpg') # doctest: +ELLIPSIS
+        PathMatch(..., args=('thumbnails/02.jpg',), kwargs={})
 
         >>> p = ExtensiblePattern("/<page:any('about', 'help')>.html")
-        >>> p.match('/about.html')
-        ((), {'page': 'about'})
+        >>> p.match('/about.html') # doctest: +ELLIPSIS
+        PathMatch(..., args=(), kwargs={'page': 'about'})
 
         >>> p = ExtensiblePattern("/entries/<id:int>")
-        >>> p.match('/entries/23')
-        ((), {'id': 23})
+        >>> p.match('/entries/23')  # doctest: +ELLIPSIS
+        PathMatch(..., args=(), kwargs={'id': 23})
 
     Others can be added by calling ``ExtensiblePattern.register_converter``
     """
@@ -307,9 +344,17 @@ class ExtensiblePattern(Pattern):
         regex = ''.join(segment.regex for segment in self.segments)
         if self.match_entire_path:
             regex += '$'
+        else:
+            regex += '(?=/|$)'
 
         self.regex = re.compile(regex)
 
+    def path_argument_info(self):
+        positional = tuple(a.converter for a in self.args if a.name is None)
+        keyword = dict((a.name, a.converter)
+                       for a in self.args if a.name is not None)
+        return (positional, keyword)
+
     def _make_segments(self, s):
         r"""
         Generate successive Segment objects from the given string.
@@ -361,16 +406,14 @@ class ExtensiblePattern(Pattern):
             return None
         groups = mo.groups()
         assert len(groups) == len(self.args), (
-                "Number of regex groups does not match expected count. "
-                "Perhaps you have used capturing parentheses somewhere? "
-                "The pattern matched was %r." % self.regex.pattern
+            "Number of regex groups does not match expected count. "
+            "Perhaps you have used capturing parentheses somewhere? "
+            "The pattern matched was %r." % self.regex.pattern
         )
 
         try:
-            groups = [
-                (segment.name, segment.converter.from_string(value))
-                  for value, segment in zip(groups, self.args)
-            ]
+            groups = [(segment.name, segment.converter.from_string(value))
+                      for value, segment in zip(groups, self.args)]
         except ValueError:
             return None
 
@@ -384,11 +427,11 @@ class ExtensiblePattern(Pattern):
 
             >>> p = ExtensiblePattern("/view/<name:str>/<revision:int>")
             >>> p.pathfor(name='important_document.pdf', revision=299)
-            '/view/important_document.pdf/299'
+            ('/view/important_document.pdf/299', [], {})
 
             >>> p = ExtensiblePattern("/view/<:str>/<:int>")
             >>> p.pathfor('important_document.pdf', 299)
-            '/view/important_document.pdf/299'
+            ('/view/important_document.pdf/299', [], {})
         """
 
         args = list(args)
@@ -397,33 +440,38 @@ class ExtensiblePattern(Pattern):
         for seg in self.segments:
             if not seg.converter:
                 result.append(seg.source)
+
             elif seg.name:
                 try:
-                    result.append(seg.converter.to_string(
-                                                    kwargs.pop(seg.name)))
+                    value = kwargs.pop(seg.name)
                 except IndexError:
                     raise URLGenerationError(
                         "Argument %r not specified for url %r" % (
                             seg.name, self.pattern
                         )
                     )
+                result.append(seg.converter.to_string(value))
+
             else:
                 try:
-                    result.append(seg.converter.to_string(args.pop(0)))
+                    value = args.pop(0)
                 except IndexError:
                     raise URLGenerationError(
                         "Not enough positional arguments for url %r" % (
                             self.pattern,
                         )
                     )
+                result.append(seg.converter.to_string(value))
+
         return ''.join(result), args, kwargs
 
     def add_prefix(self, prefix):
-        return self.__class__(join_path(prefix, self.pattern))
+        return self.__class__(join_path(prefix, self.pattern),
+                              self.match_entire_path)
 
     @classmethod
     def register_converter(cls, name, converter):
-        """
+        r"""
         Register a preset pattern for later use in URL patterns.
 
         Example usage::
@@ -436,8 +484,9 @@ class ExtensiblePattern(Pattern):
             ...         return date(*strptime(s, '%d%m%Y')[:3])
             ...
             >>> ExtensiblePattern.register_converter('date', DateConverter)
-            >>> ExtensiblePattern('/<:date>').match('/01011970')
-            ((datetime.date(1970, 1, 1),), {})
+            >>> ExtensiblePattern('/<:date>')\
+            ...      .match('/01011970')  # doctest:+ELLIPSIS
+            PathMatch(..., args=(datetime.date(1970, 1, 1),), kwargs={})
         """
         cls.preset_patterns += ((name, converter),)
 
@@ -457,7 +506,8 @@ class Route(object):
     pattern_class = ExtensiblePattern
 
     def __init__(self, pattern, methods, view, view_kwargs=None,
-                 name=None, predicate=None, decorators=None, **kwargs):
+                 name=None, predicate=None, decorators=None,
+                 **kwargs):
 
         """
         :param pattern:     A string that can be compiled into a path pattern
@@ -497,7 +547,7 @@ class Route(object):
         If you generate a URL for the view function ``display_image``, the
         the first declared route will always win, in this case::
 
-            >>> urlfor(display_image)
+            >>> urlfor(display_image)  # doctest: +SKIP
             'http://localhost/image'
 
         To generate URLs for the thumbnail route in this example, you must
@@ -506,9 +556,9 @@ class Route(object):
             Route('/image', display_image, name='image', ...)
             Route('/thumbnail', display_image, name='image-thumbnail', ...)
 
-            >> urlfor('image')
+            >> urlfor('image')  # doctest: +SKIP
             'http://localhost/image'
-            >> urlfor('image-thumbnail')
+            >> urlfor('image-thumbnail')  # doctest: +SKIP
             'http://localhost/thumbnail'
 
         """
@@ -529,31 +579,56 @@ class Route(object):
             # Colons are reserved to act as separators
             raise ValueError("Route names cannot contain ':'")
 
+        self.name = name
+        self.predicate = predicate
+        self.decorators = decorators or []
+
+        #: Default values to use for path generation
+        self.routed_args_default = {}
+
         self.pattern = pattern
         self.methods = set(methods)
+        self.instance = None
+        # Cached reference to the view function
+        self._view = None
+        # Cached reference to the decorated view function
+        self._dview = None
+
+        p_args, p_kwargs = pattern.path_argument_info()
+        for k in p_kwargs:
+            default = kwargs.pop(k + '_default', _marker)
+            if default is not _marker:
+                self.routed_args_default[k] = default
+
         self.view_kwargs = dict(view_kwargs or {}, **kwargs)
         for argname, arg in self.view_kwargs.items():
             if isinstance(arg, RouteKwarg):
                 arg.configure(self, argname)
 
-        self.name = name
-        self.predicate = predicate
-        self.decorators = decorators or []
-        self.set_view(view)
+        self.viewspec = view
 
     def __str__(self):
+        if self.methods == ALL_METHODS:
+            methods = '*'
+        else:
+            methods = ' '.join(self.methods)
+
         return '%s %s => %r\n' % (
-            ' '.join(self.methods),
+            methods,
             str(self.pattern),
-            self.original_view
+            self.viewspec
         )
 
     def __repr__(self):
+        if self.methods == ALL_METHODS:
+            methods = '*'
+        else:
+            methods = ','.join(self.methods)
         return '<%s %s %r %s>' % (
             self.__class__.__name__,
-            ','.join(self.methods),
+            methods,
             str(self.pattern),
-            self.original_view
+            self.viewspec
         )
 
     def match(self, path, method):
@@ -561,21 +636,45 @@ class Route(object):
             return None
         return self.pattern.match(path)
 
-    def set_view(self, view):
-        decorated_view = view
-        if not isinstance(view, string_types):
-            for d in (self.decorators or []):
-                decorated_view = d(decorated_view)
-
-        self.original_view = view
-        self.view = decorated_view
-
-    def resolve_view(self, instance):
+    def getview(self):
+        """\
+        Return the raw view callable.
         """
-        Resolve a view spec string to the actual callable.
+        if self._view:
+            return self._view
+        uview = self.viewspec
+        if isinstance(uview, string_types):
+            uview = getattr(self.instance, uview)
+
+        self._view = uview
+        return uview
+
+    def getdecoratedview(self):
+        """\
+        Return the view callable decorated with any decorators defined in the
+        route
         """
-        if isinstance(self.original_view, string_types):
-            self.set_view(getattr(instance, self.original_view))
+        if self._dview:
+            return self._dview
+        dview = self.getview()
+        for d in (self.decorators or []):
+            dview = d(dview)
+        self._dview = dview
+        return dview
+
+    def bindto(self, instance):
+        """\
+        Return copy of the route bound to a given instance.
+
+        Use this when traversing view classes
+        """
+        copy = Route.__new__(self.__class__)
+        copy.__dict__ = self.__dict__.copy()
+        copy._setinstance(instance)
+        return copy
+
+    def _setinstance(self, instance):
+        self.instance = instance
 
     def add_prefix(self, path):
         """
@@ -587,7 +686,7 @@ class Route(object):
                                  pattern=self.pattern.add_prefix(path))
         return newroute
 
-    def path(self, *args, **kwargs):
+    def path(self, request=None, *args, **kwargs):
         """\
         Return a tuple of URL path component corresponding to the route and
         remaining positional and keyword arguments
@@ -595,6 +694,12 @@ class Route(object):
         Positional and keywords arguments are forwarded to the ``pathfor``
         method of the pattern.
         """
+        for k in self.routed_args_default:
+            if k not in kwargs:
+                v = self.routed_args_default[k]
+                if callable(v):
+                    v = v(request)
+                kwargs[k] = v
         return self.pattern.pathfor(*args, **kwargs)
 
     def route_keys(self):
@@ -603,11 +708,12 @@ class Route(object):
         """
         if self.name:
             yield self.name
-        yield self.original_view
+        view = self.getview()
+        yield view
 
         # Also return the underlying function object for python2 unbound
         # methods
-        func_ob = getattr(self.original_view, '__func__', None)
+        func_ob = getattr(view, '__func__', None)
         if func_ob is not None:
             yield func_ob
 
@@ -646,18 +752,13 @@ class RouteCollection(object):
     def __init__(self, routes=None):
         self.__routes__ = []
         self.__routed_views__ = {}
-        if routes:
+        if routes is not None:
             for item in routes:
                 self.add_route(item)
 
     def add_route(self, route):
         self.__routes__.append(route)
-
-        for item in route.route_keys():
-            if item not in self.__routed_views__:
-                self.__routed_views__[item] = route
-
-        self._add_url_method(route.original_view, route)
+        self._add_url_method(route.getview(), route)
 
     def _add_url_method(self, func, route):
         """\
@@ -674,7 +775,7 @@ class RouteCollection(object):
             # See http://www.python.org/dev/peps/pep-0232/
             pass
 
-    def pathfor(self, viewspec, *args, **kwargs):
+    def pathfor(self, viewspec, request=None, *args, **kwargs):
         """\
         Return the path component of the url for the given view name/function
         spec.
@@ -688,17 +789,25 @@ class RouteCollection(object):
             delegated_route = self.routefor(viewspec)
 
             p1, remainder_args, remainder_kwargs = \
-                    delegated_route.path(*args, **kwargs)
+                delegated_route.path(*args, **kwargs)
 
             factory_args = args[:-len(remainder_args)]
-            factory_kwargs = {k: v for k, v in kwargs.items()
-                                   if k not in remainder_kwargs}
+            factory_kwargs = dict((k, v)
+                                  for k, v in kwargs.items()
+                                  if k not in remainder_kwargs)
+            for k in delegated_route.routed_args_default:
+                if k not in factory_kwargs:
+                    v = delegated_route.routed_args_default[k]
+                if callable(v):
+                    v = v(request)
+                factory_kwargs[k] = v
             rc = delegated_route.routecollectionfactory(*factory_args,
                                                         **factory_kwargs)
-            p2 = rc.pathfor(remainder, *args, **kwargs)
+            p2 = rc.pathfor(remainder, request=request, *args, **kwargs)
             return p1 + p2
 
-        return self.routefor(viewspec).path(*args, **kwargs)[0]
+        return self.routefor(viewspec).path(request=request,
+                                            *args, **kwargs)[0]
 
     def routefor(self, viewspec):
         """
@@ -713,37 +822,47 @@ class RouteCollection(object):
         :param viewspec: a view callable or a string in the form
                          'package.module.viewfunction'
         """
+        try:
+            return self.__routed_views__[viewspec]
+        except KeyError:
+            pass
+
+        for route in self.__routes__:
+            for k in route.route_keys():
+                self.__routed_views__.setdefault(k, route)
 
         try:
             return self.__routed_views__[viewspec]
         except KeyError:
-            if not isinstance(viewspec, string_types):
+            pass
+
+        if not isinstance(viewspec, string_types):
+            raise RouteNotFound(viewspec)
+
+        modname, symbols = viewspec, []
+        while True:
+            try:
+                modname, sym = modname.rsplit('.', 1)
+            except ValueError:
                 raise RouteNotFound(viewspec)
 
-            modname, symbols = viewspec, []
-            while True:
+            symbols.append(sym)
+            module = sys.modules.get(modname, None)
+            if module:
+                ob = module
+                for s in reversed(symbols):
+                    ob = getattr(ob, s)
                 try:
-                    modname, sym = modname.rsplit('.', 1)
-                except ValueError:
-                    raise RouteNotFound(viewspec)
-
-                symbols.append(sym)
-                module = sys.modules.get(modname, None)
-                if module:
-                    ob = module
-                    for s in reversed(symbols):
-                        ob = getattr(ob, s)
+                    route = self.__routed_views__[ob]
+                except KeyError:
+                    # An unbound class method? (python 2 only)
                     try:
-                        route = self.__routed_views__[ob]
-                    except KeyError:
-                        # An unbound class method? (python 2 only)
-                        try:
-                            route = self.__routed_views__[ob.__func__]
-                        except (AttributeError, KeyError):
-                            raise RouteNotFound(viewspec)
+                        route = self.__routed_views__[ob.__func__]
+                    except (AttributeError, KeyError):
+                        raise RouteNotFound(viewspec)
 
-                    self.__routed_views__[viewspec] = route
-                    return route
+                self.__routed_views__[viewspec] = route
+                return route
 
     def get_routes(self, path, method):
         """
@@ -830,6 +949,9 @@ class RouteCollection(object):
         return route
 
     def route_wsgi(self, path, wsgiapp, *args, **kwargs):
+        """
+        Route requests to ``path`` to the given WSGI application
+        """
 
         from fresco import context
 
@@ -863,7 +985,7 @@ class RouteCollection(object):
         """
         routes = list(r.add_prefix(path) for r in views.__routes__)
         for r in routes:
-            r.resolve_view(views)
+            r = r.bindto(views)
             self.add_route(r)
 
     def delegate(self, path, app, dynamic=False, *args, **kwargs):
@@ -899,26 +1021,18 @@ class DelegateRoute(Route):
 
     def _dynamic_routecollectionfactory(self, *args, **kwargs):
         """\
-        Return the RouteCollection  responsible for paths under this route and
-        the remaining positional+keyword args
+        Return the RouteCollection responsible for paths under this route
         """
 
-        cached = context.get(self)
-        if cached:
-            return cached
-
-        routes = self.view(*args, **kwargs)
+        routes = self.getdecoratedview()(*args, **kwargs)
 
         if isinstance(routes, RouteCollection):
             return routes
 
-        for r in routes.__routes__:
-            r.resolve_view(routes)
-
-        return RouteCollection(routes.__routes__)
+        return RouteCollection(r.bindto(routes) for r in routes.__routes__)
 
     def _static_routecollectionfactory(self, *args, **kwargs):
-        return self.view
+        return self.getview()
 
 
 class RouteKwarg(object):
@@ -977,14 +1091,12 @@ class RequestArg(RouteKwarg):
     #: input
     converter_exceptions = (ValueError, TypeError)
 
-    _marker = object()
-
     def __init__(self, converter=None, key=None,
                  default=_marker, exception=BadRequest):
 
         self.formkey = key
         self.default = default
-        self.required = default is self._marker
+        self.required = default is _marker
         self.exception = exception
         self.is_list = isinstance(converter, list)
         if self.is_list:
@@ -1006,7 +1118,7 @@ class RequestArg(RouteKwarg):
             value = self.getter(self.source(request))
         except KeyError:
             if self.required:
-                raise self.exception('No value provided for %s' %\
+                raise self.exception('No value provided for %s' %
                                      (self.formkey,))
             return self.default
 
@@ -1014,7 +1126,7 @@ class RequestArg(RouteKwarg):
             if self.converter is not None:
                 value = self.converter(value)
         except self.converter_exceptions:
-            raise self.exception('%r is not a valid value for %s' %\
+            raise self.exception('%r is not a valid value for %s' %
                                  (value, self.formkey,))
 
         return value
