@@ -19,7 +19,7 @@ def serve_static_file(path, default_charset="ISO-8859-1",
     """
     Serve a static file located at ``path``.
 
-    :returns: A :class:`fresco.request.Request` response for the file at
+    :returns: A :class:`~fresco.response.Response` object for the file at
               ``path``
     """
 
