@@ -165,13 +165,13 @@ class SizeLimitedInput(Reader, Seeker, ReadlinesMixin):
         >>> s.seek(0)
         0L
         >>> s.read()
-        b'the rain in spain\nfalls '
+        'the rain in spain\nfalls '
         >>> s.seek(0)
         0L
         >>> s.readline()
-        b'the rain in spain\n'
+        'the rain in spain\n'
         >>> s.readline()
-        b'falls '
+        'falls '
     """
 
     def __init__(self, raw, length):
@@ -235,13 +235,13 @@ class DelimitedInput(Reader, ReadlinesMixin):
         >>> s = BytesIO(b'one--two--three')
         >>> p = PutbackInput(s)
         >>> DelimitedInput(p, b'--').read()
-        b'one'
+        'one'
         >>> DelimitedInput(p, b'--').read()
-        b'two'
+        'two'
         >>> DelimitedInput(p, b'--').read()
-        b'three'
+        'three'
         >>> DelimitedInput(p, b'--').read()
-        b''
+        ''
 
     """
 
