@@ -2,11 +2,13 @@
 Utility functions for URL building and manipulation
 """
 
+import posixpath
+from fresco.compat import quote_plus, urlparse, urlunparse, ustring_type
+
 import fresco
 from fresco.multidict import MultiDict
-from urllib import quote_plus
 
-__all__ = 'join_path', 'strip_trailing_slashes'
+__all__ = 'join_path', 'url_join', 'strip_trailing_slashes', 'normpath'
 
 
 def join_path(a, b):
@@ -45,6 +47,44 @@ def join_path(a, b):
     return a + '/' + b
 
 
+def url_join(base, rel):
+    """
+    Examples::
+
+        >>> from fresco.util.urls import url_join
+        >>> url_join('http://example.org/', 'http://example.com/')
+        'http://example.com/'
+        >>> url_join('http://example.com/', '../styles/main.css')
+        'http://example.com/styles/main.css'
+        >>> url_join('http://example.com/subdir/', '../styles/main.css')
+        'http://example.com/styles/main.css'
+        >>> url_join('http://example.com/login', '?error=failed+auth')
+        'http://example.com/login?error=failed+auth'
+        >>> url_join('http://example.com/login', 'register')
+        'http://example.com/register'
+    """
+    prel = urlparse(rel)
+
+    # Link is already absolute, return it unchanged
+    if prel.scheme:
+        return rel
+
+    pbase = urlparse(base)
+    path = pbase.path
+    if prel.path:
+        path = normpath(posixpath.join(posixpath.dirname(pbase.path),
+                                       prel.path))
+
+    return urlunparse((
+        pbase.scheme,
+        pbase.netloc,
+        path,
+        prel.params,
+        prel.query,
+        prel.fragment
+    ))
+
+
 def strip_trailing_slashes(path):
     """\
     Remove trailing slashes from the given path.
@@ -116,20 +156,20 @@ def make_query(data=None, separator=';', charset=None, **kwargs):
 
     Basic usage::
 
-        >>> make_query({'search': u'foo bar', 'type': u'full'})
+        >>> make_query({'search': 'foo bar', 'type': u'full'})
         'search=foo+bar;type=full'
 
-        >>> make_query(search=u'foo', type=u'quick')
+        >>> make_query(search='foo', type=u'quick')
         'search=foo;type=quick'
 
     Changing the separator to '&'::
 
-        >>> make_query({'search': u'foo', 'type': u'full'}, separator='&')
+        >>> make_query({'search': 'foo', 'type': u'full'}, separator='&')
         'search=foo&type=full'
 
     Multiple values can be provided per key::
 
-        >>> make_query(search=[u'foo', u'bar'])
+        >>> make_query(search=['foo', u'bar'])
         'search=foo;search=bar'
 
     Exclude values by setting them to ``None``::
@@ -137,16 +177,20 @@ def make_query(data=None, separator=';', charset=None, **kwargs):
         >>> make_query(x=1, y=None)
         'x=1'
 
+    :param data: data for query string
+    :param separator: separator used between each key value pair
+    :param charset: encoding to be used for unicode values
+    :rtype: str
     """
     if isinstance(data, MultiDict):
-        items = data.allitems()
+        items = list(data.allitems())
     elif isinstance(data, dict):
-        items = data.items()
+        items = list(data.items())
     elif data is None:
         items = []
     else:
         items = list(data)
-    items += kwargs.items()
+    items += list(kwargs.items())
 
     if charset is None:
         charset = fresco.DEFAULT_CHARSET
@@ -155,47 +199,32 @@ def make_query(data=None, separator=';', charset=None, **kwargs):
     items.sort()
     items = _repeat_keys(items)
     items = ((k, v) for k, v in items if v is not None)
-    return separator.join([
-        _qs_frag(k, v, charset=charset) for k, v in items
-    ])
+    return separator.join(_qs_frag(k, v, charset=charset) for k, v in items)
 
 
 def _qs_frag(key, value, charset=None):
-    u"""
-    Return a fragment of a query string in the format 'key=value'.
+    """\
+    Return a fragment of a query string in the format 'key=value'::
 
-    >>> _qs_frag('search-by', 'author, editor')
-    'search-by=author%2C+editor'
+        >>> _qs_frag('search-by', 'author, editor')
+        'search-by=author%2C+editor'
 
     If no encoding is specified, unicode values are encoded using the character
     set specified by ``fresco.DEFAULT_CHARSET``.
+
+    :rtype: str
     """
     if charset is None:
         charset = fresco.DEFAULT_CHARSET
 
-    return quote_plus(_make_bytestr(key, charset)) \
-            + '=' \
-            + quote_plus(_make_bytestr(value, charset))
+    key = ustring_type(key).encode(charset)
+    value = ustring_type(value).encode(charset)
 
+    return quote_plus(key) + '=' + quote_plus(value)
 
-def _make_bytestr(ob, charset):
-    u"""
-    Return a byte string conversion of the given object. If the object is a
-    unicode string, encode it with the given encoding.
-
-    Example::
-
-        >>> _make_bytestr(1, 'utf-8')
-        '1'
-        >>> _make_bytestr(u'a', 'utf-8')
-        'a'
-    """
-    if isinstance(ob, unicode):
-        return ob.encode(charset)
-    return str(ob)
 
 def _repeat_keys(iterable):
-    u"""
+    """
     Return a list of ``(key, scalar_value)`` tuples given an iterable
     containing ``(key, iterable_or_scalar_value)``.
 
@@ -212,7 +241,7 @@ def _repeat_keys(iterable):
     """
 
     for key, value in iterable:
-        if isinstance(value, basestring):
+        if isinstance(value, (ustring_type, bytes)):
             value = [value]
         else:
             try:
